Package: OUTRIDER
Title: OUTRIDER - OUTlier in RNA-Seq fInDER
Type: Package
Version: 1.31.0
Date: 2026-02-10
URL: https://github.com/gagneurlab/OUTRIDER
BugReports: https://github.com/gagneurlab/OUTRIDER/issues
Authors@R: c(
    person("Felix", "Brechtmann", role=c("aut"), 
            email="brechtma@in.tum.de", comment=c(ORCID="0000-0002-0110-152X")),
    person("Christian", "Mertes", role=c("aut", "cre"), 
            email="mertes@in.tum.de", comment=c(ORCID="0000-0002-1091-205X")),
    person("Agne", "Matuseviciute", role=c("aut")),
    person(c("Michaela", "Fee"), "Müller", role=c("ctb")),
    person("Andrea", "Raithel", role=c("ctb")),
    person("Vicente", "Yepez", role=c("aut"), email="yepez@in.tum.de",
            comment=c(ORCID="0000-0001-7916-3643")),
    person("Julien", "Gagneur", role=c("aut"), email="gagneur@in.tum.de",
            comment=c(ORCID="0000-0002-8924-8365")))
Description: Identification of aberrant gene expression in RNA-seq data.
    Read count expectations are modeled by an autoencoder to control for
    confounders in the data. Given these expectations, the RNA-seq read counts
    are assumed to follow a negative binomial distribution with a gene-specific
    dispersion. Outliers are then identified as read counts that significantly 
    deviate from this distribution. Furthermore, OUTRIDER provides useful 
    plotting functions to analyze and visualize the results.
VignetteBuilder: knitr
biocViews: ImmunoOncology, RNASeq, Transcriptomics, Alignment,
        Sequencing, GeneExpression, Genetics
License: file LICENSE
NeedsCompilation: yes
Encoding: UTF-8
RoxygenNote: 7.3.2
Depends: R (>= 3.6), BiocParallel, GenomicFeatures,
        SummarizedExperiment, methods
Imports: BBmisc, BiocGenerics, data.table, DESeq2 (>= 1.16.1),
        generics, GenomicRanges, ggplot2, ggrepel, graphics, grDevices,
        heatmaply, IRanges, matrixStats, pcaMethods, pheatmap, plotly,
        plyr, pracma, PRROC, RColorBrewer, reshape2, RMTstat,
        S4Vectors, scales, splines, stats, txdbmaker, utils
Suggests: testthat, knitr, rmarkdown, BiocStyle,
        TxDb.Hsapiens.UCSC.hg19.knownGene, org.Hs.eg.db, RMariaDB,
        AnnotationDbi, beeswarm, covr, GenomeInfoDb, ggbio, biovizBase
LinkingTo: Rcpp, RcppArmadillo
Collate: package-OUTRIDER.R class-OutriderDataSet.R AllGenerics.R
        inputCheckerFunctions.R helperFunctions.R getNSetterFuns.R
        getNSetterFunsInternal.R autoencoder.R fitNB.R ZscoreMatrix.R
        method-evaluation.R method-counts.R method-estimateBestQ.R
        method-plot.R method-results.R pValMatrix.R filterExpression.R
        OUTRIDER.R sizeFactor.R RcppExports.R updateE.R updateD.R
        updateTheta.R PCAcorrections.R thetaMethodOfMoments.R
        controlForConfounders.R lossNGradientRVersion.R
git_url: https://git.bioconductor.org/packages/OUTRIDER
git_branch: devel
git_last_commit: f5f7a6b
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.24
Date/Publication: 2026-04-28
Packaged: 2026-04-29 00:18:11 UTC; biocbuild
Author: Felix Brechtmann [aut] (ORCID: <https://orcid.org/0000-0002-0110-152X>),
  Christian Mertes [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-1091-205X>),
  Agne Matuseviciute [aut],
  Michaela Fee Müller [ctb],
  Andrea Raithel [ctb],
  Vicente Yepez [aut] (ORCID: <https://orcid.org/0000-0001-7916-3643>),
  Julien Gagneur [aut] (ORCID: <https://orcid.org/0000-0002-8924-8365>)
Maintainer: Christian Mertes <mertes@in.tum.de>
