# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

predictMatY <- function(x, E, D, b) {
    .Call('_OUTRIDER_predictMatY', PACKAGE = 'OUTRIDER', x, E, D, b)
}

predictMatC <- function(x, E, D, b, sf) {
    .Call('_OUTRIDER_predictMatC', PACKAGE = 'OUTRIDER', x, E, D, b, sf)
}

truncLogLiklihoodD <- function(par, H, k, sf, exclusionMask, theta, thetaC) {
    .Call('_OUTRIDER_truncLogLiklihoodD', PACKAGE = 'OUTRIDER', par, H, k, sf, exclusionMask, theta, thetaC)
}

gradientD <- function(par, H, k, sf, exclusionMask, theta, thetaC) {
    .Call('_OUTRIDER_gradientD', PACKAGE = 'OUTRIDER', par, H, k, sf, exclusionMask, theta, thetaC)
}

truncLogLiklihoodE <- function(e, D, k, b, x, sf, theta, exclusionMask, thetaC) {
    .Call('_OUTRIDER_truncLogLiklihoodE', PACKAGE = 'OUTRIDER', e, D, k, b, x, sf, theta, exclusionMask, thetaC)
}

gradientE <- function(e, D, k, b, x, sf, theta, exclusionMask, thetaC) {
    .Call('_OUTRIDER_gradientE', PACKAGE = 'OUTRIDER', e, D, k, b, x, sf, theta, exclusionMask, thetaC)
}

